;; byte-compiled version will be loaded by yerushalmi.dba

;; TODO
;; 
;; single-key report?
;; tie to calendar to fill in today's info in new record?  this is a bit
;; of a bear because of skipping Yom Kippur and Tisha B'Av on the Hebrew
;; calendar..

(defvar yerushalmi-mblatt [68 37 34 44 31 59 26 33 28 20 13 
                      92 65 71 22 22 42 26 26 33 34 19 22 
                      85 72 47 40 47 54 48 
                      44 37 34 57 9 44 37 19 13])

(defvar yerushalmi-delete-this-record nil)
(defvar yerushalmi-go-back-to-field -1)

(setq dbf-every-change-function 'yerushalmi-check-change-validity)
(setq dbf-enter-field-hook 'yerushalmi-check-special-actions)
(setq db-new-record-function 'yerushalmi-new-entry)

(defun yerushalmi-new-entry (record database)
"Set up a new yerushalmi record defaulting to today and prompting the rest"
(if (>= (database-no-of-records database) 3108) 
 (progn
  (setq yerushalmi-delete-this-record t)
  (read-from-minibuffer "Database full  cannot add new record.  Press ENTER")
;; this next doesn't seem to work
  (db-view-mode))
(progn
 (record-set-field record 'ldate 
	(parse-date-string (calendar-date-string (calendar-current-date))) database)
 (record-set-field record 'yerushalmi 1 database)
 (record-set-field record 'rashi 0 database)
 (record-set-field record 'tosafos 0 database)
 (record-set-field record 'other 0 database)
 (setq yerushalmi-good-value nil)
 (while (equal nil yerushalmi-good-value)
  (progn
   (yerushalmi-masechtot-help)
   (while (not (yerushalmi-valid-masechta (setq yerushalmi-masechta 
      (read-from-minibuffer "Enter Masechta: ")))))
   (switch-to-buffer "yerushalmi.dat")
   (yerushalmi-get-blatt-limits yerushalmi-masechta)
   (while (not (yerushalmi-valid-daf (setq yerushalmi-daf (string-to-int 
      (read-from-minibuffer 
       (concat "Enter Daf (" yerushalmi-min-blatt "-" yerushalmi-max-blatt "): ")
     )))) ))
   (while (not (yerushalmi-valid-amud (setq yerushalmi-amud 
      (read-from-minibuffer "Enter Amud in lowercase  a or b: ")))))
   (if (not (yerushalmi-dupe-check yerushalmi-masechta yerushalmi-daf yerushalmi-amud))
    (setq yerushalmi-good-value t))))
   (kill-buffer "*Masechtot*")
   (switch-to-buffer "yerushalmi.dat")
   (delete-other-windows)
   (record-set-field record 'masechta yerushalmi-masechta database)
   (record-set-field record 'daf yerushalmi-daf database)
   (record-set-field record 'amud yerushalmi-amud database)
)))

(defun yerushalmi-check-change-validity (cfield cold cnew)
;; need a dupe check for daf  amud  masechta plus daf validity
(setq yerushalmi-check-ok t)
(if (eq cfield 'daf)
   (progn
    (yerushalmi-get-blatt-limits (dbf-displayed-record-field 'masechta))
    (if	(not (yerushalmi-valid-daf cnew))
      (setq yerushalmi-check-ok nil))))
(if (or  
	(eq cfield 'daf)
        (eq cfield 'masechta)
        (eq cfield 'amud))
    (progn
	(if (yerushalmi-dupe-check (dbf-displayed-record-field 'masechta)
                               (dbf-displayed-record-field 'daf)
                               (dbf-displayed-record-field 'amud))
	    (setq yerushalmi-check-ok nil))))
(if (equal nil yerushalmi-check-ok)
     (progn
        (setq yerushalmi-go-back-to-field (fieldname->fieldnumber cfield dbc-database))
	(dbf-displayed-record-set-field cfield cold)
        (message "Your entry is invalid  retaining old value.")))
)

(defun yerushalmi-dupe-check (tmasechta tdaf tamud)
(maprecords-macro
 (progn
 (setq yerushalmi-found-dupe nil)
 (progn
   (if (and (equal tdaf (record-field maprecords-record 'daf dbc-database))
            (equal tmasechta (record-field maprecords-record 'masechta dbc-database))
	    (equal tamud (record-field maprecords-record 'amud dbc-database)))
       (progn
	(read-from-minibuffer "Duplicate records not allowed.  Press ENTER.")
	(setq yerushalmi-found-dupe t)
        (maprecords-break))
 ))) dbc-database)
 (if yerushalmi-found-dupe
     t
     nil)
)

(defun yerushalmi-valid-masechta (tmasechta)
  (if (not (= -1 (yerushalmi-find-masechta tmasechta)))
   t
   (progn
    (read-from-minibuffer "Invalid Masechta  press ENTER to try again.")
    nil )
))

(defun yerushalmi-find-masechta (tmasechta)
 (setq yerushalmi-i 0)
 (setq yerushalmi-found -1)
 (while (and (< yerushalmi-i (length yerushalmi-mnames))
	     (= yerushalmi-found -1))
  (progn
    (if (equal (elt yerushalmi-mnames yerushalmi-i) tmasechta)
        (setq yerushalmi-found yerushalmi-i)
	(setq yerushalmi-i (+ 1 yerushalmi-i)))))
 (setq yerushalmi-i yerushalmi-found)
)

(defun yerushalmi-get-blatt-limits (tmasechta)
 (setq yerushalmi-blatt-index (yerushalmi-find-masechta tmasechta))
 (setq yerushalmi-min-blatt 1)  ;; 1 not 2
 (setq yerushalmi-max-blatt (aref yerushalmi-mblatt yerushalmi-blatt-index))
)

(defun yerushalmi-valid-daf (tdaf)
;; yerushalmi-get-blatt-limits must have been called first
 (if (and  (integerp tdaf)
           (>= tdaf yerushalmi-min-blatt)
	   (<= tdaf yerushalmi-max-blatt))    
   t
 (progn
  (read-from-minibuffer "Invalid value of Daf for this Masechta - press ENTER.")
    nil))
)

(defun yerushalmi-valid-amud (tamud)
 (if 
       (or (equal tamud "a") 
           (equal tamud "b"))
        t
 (progn
  (read-from-minibuffer "Amud must be 'a' or 'b' (lower case); press ENTER to try again.")
	nil))
)

(defun yerushalmi-check-special-actions ()
;; avoid infinite recursion
 (setq dbf-enter-field-hook nil)
 (if (not (= -1 yerushalmi-go-back-to-field))
  (progn
;; yes  this does work!
   (db-last-field)
   (if (not (= 0 yerushalmi-go-back-to-field))
     (db-next-field yerushalmi-go-back-to-field))
   (setq yerushalmi-go-back-to-field -1)))
 (if (equal t yerushalmi-delete-this-record)
  (progn
   (setq yerushalmi-delete-this-record nil)
   (db-delete-record 1)))
 (setq dbf-enter-field-hook 'yerushalmi-check-special-actions)
)

(defun yerushalmi-masechtot-help ()
 (if (get-buffer "*Masechtot*")
   (kill-buffer "*Masechtot*"))
 (delete-other-windows)
 (split-window)
 (get-buffer "*Masechtot*")
 (switch-to-buffer "*Masechtot*")
 (insert (prin1-to-string yerushalmi-mnames))
 (goto-char (point-min))
 (while (search-forward "\"" nil t)
   (replace-match "" nil t))
 (goto-char (point-min))
 (while (search-forward "(" nil t)
   (replace-match "" nil t))
 (goto-char (point-min))
 (while (search-forward ")" nil t)
   (replace-match "" nil t))
 (goto-char (point-min))
 (while (search-forward " " nil t 8)
   (insert "\n"))
 (goto-char (point-min))
 (insert "Valid Masechtot:\n\n")
)

(defun yerushalmi-report () 
"Give all sorts of stats on learning yerushalmi."
(interactive)
;; We need to build a table of masechtot studied  and we can only do
;; so by scanning the database and keeping track.  We keep other elements
;; for total times studied per masechta  and sum to shas.
 (switch-to-buffer "yerushalmi.dat")
 (db-sort 1)
 (setq yerushalmi-masechtot-learned (make-vector (length yerushalmi-mblatt) nil))
 (setq yerushalmi-yerushalmi-learned (make-vector (length yerushalmi-mblatt) 0))
 (setq yerushalmi-rashi-learned (make-vector (length yerushalmi-mblatt) 0))
 (setq yerushalmi-tosafot-learned (make-vector (length yerushalmi-mblatt) 0))
 (setq yerushalmi-other-learned (make-vector (length yerushalmi-mblatt) 0))
 (setq yerushalmi-masechta-counter 0)
 (setq yerushalmi-amud-learned 0)

 (maprecords-macro
  (progn
   (setq yerushalmi-amud-learned (+ 1 yerushalmi-amud-learned))
;; do we have an entry yet for this record's masechta?   
   (setq yerushalmi-i 0)
   (setq yerushalmi-loc -1)

   (while (and 
             (<= yerushalmi-i yerushalmi-masechta-counter)
	     (= -1 yerushalmi-loc))
    (progn
     (if (equal (aref yerushalmi-masechtot-learned yerushalmi-i) 
              (record-field maprecords-record 'masechta dbc-database))
        (setq yerushalmi-loc yerushalmi-i))
     (setq yerushalmi-i (+ 1 yerushalmi-i))))

   (if (= -1 yerushalmi-loc)
    (progn
     (setq yerushalmi-loc yerushalmi-masechta-counter)
     (aset yerushalmi-masechtot-learned yerushalmi-loc (record-field maprecords-record 'masechta dbc-database))
     (setq yerushalmi-masechta-counter (+ 1 yerushalmi-masechta-counter))))

   (aset yerushalmi-yerushalmi-learned yerushalmi-loc 
     (+ (aref yerushalmi-yerushalmi-learned yerushalmi-loc)
        (record-field maprecords-record 'yerushalmi dbc-database)
     ))
   (aset yerushalmi-rashi-learned yerushalmi-loc
     (+ (aref yerushalmi-rashi-learned yerushalmi-loc) 
        (record-field maprecords-record 'rashi dbc-database)
     ))
   (aset yerushalmi-tosafot-learned yerushalmi-loc
     (+ (aref yerushalmi-tosafot-learned yerushalmi-loc) 
        (record-field maprecords-record 'tosafos dbc-database)
     ))
   (aset yerushalmi-other-learned yerushalmi-loc
     (+ (aref yerushalmi-other-learned yerushalmi-loc) 
        (record-field maprecords-record 'other dbc-database)
     ))
  ) dbc-database)

;; show info
 (if (get-buffer "*Gemara Learning Statistics*")
   (kill-buffer "*Gemara Learning Statistics*"))
 (delete-other-windows)
 (split-window)
 (get-buffer "*Gemara Learning Statistics*")
 (switch-to-buffer "*Gemara Learning Statistics*")
 (delete-other-windows)
 (insert "B\"H\n")
 (insert "        ***** Gemara Yerushalmi Learning Statistics *****\n\n")
 (insert "You have made entries for " (int-to-string yerushalmi-amud-learned) " amud")
 (insert " in " (int-to-string yerushalmi-masechta-counter) " different masechtot.\n\n")
 (insert "Total times amudim learned  by masechta:\n\n")
 (insert "                    Gemara\tRashi\tTosafos\tOther\n\n")
 (setq yerushalmi-i 0)
 (setq yerushalmi-yerushalmi-total 0)
 (setq yerushalmi-rashi-total 0)
 (setq yerushalmi-tosafot-total 0)
 (setq yerushalmi-other-total 0)
 (while (< yerushalmi-i yerushalmi-masechta-counter)
  (progn
   (insert (format "%-20s" (aref yerushalmi-masechtot-learned yerushalmi-i) ))
   (insert (int-to-string (aref yerushalmi-yerushalmi-learned yerushalmi-i)) "\t\t")
   (insert (int-to-string (aref yerushalmi-rashi-learned yerushalmi-i)) "\t")
   (insert (int-to-string (aref yerushalmi-tosafot-learned yerushalmi-i)) "\t")
   (insert (int-to-string (aref yerushalmi-other-learned yerushalmi-i)) "\t")
   (insert "\n")
   (setq yerushalmi-yerushalmi-total (+ yerushalmi-yerushalmi-total (aref yerushalmi-yerushalmi-learned yerushalmi-i)))
   (setq yerushalmi-rashi-total (+ yerushalmi-rashi-total (aref yerushalmi-rashi-learned yerushalmi-i)))
   (setq yerushalmi-tosafot-total (+ yerushalmi-tosafot-total (aref yerushalmi-tosafot-learned yerushalmi-i)))
   (setq yerushalmi-other-total (+ yerushalmi-other-total (aref yerushalmi-other-learned yerushalmi-i)))
 (setq yerushalmi-i (+ 1 yerushalmi-i)) ))

;; now print summary stats
 (insert "\nYou have learned Gemara " (int-to-string yerushalmi-yerushalmi-total) " times,\n")
 (insert   "                  Rashi " (int-to-string yerushalmi-rashi-total) " times,\n")
 (insert   "                Tosefot " (int-to-string yerushalmi-tosafot-total) " times,\n")
 (insert   "      and other sources " (int-to-string yerushalmi-other-total) " times.\n")  

)

(defun yerushalmi-sort ()
"Sort the database by tractate ordering."
 (interactive)
 (save-excursion
  (switch-to-buffer "yerushalmi.dat")
  (database-sort dbc-database 'yerushalmi-tractate-sort)
  (dbf-finished-sorting))
)
